// -*- mode: Rust -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

use crate::private::Ping;
use once_cell::sync::Lazy;

#[allow(non_upper_case_globals)]
/// This is a ping representing single events triggered by the messaging system
/// and captures some pings from About:Welcome, ASRouter, and other corners.
pub static messaging_system: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "messaging-system",
        false,
        false,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// Newtab-related instrumentation.
/// Can be disabled via the `newtabPingEnabled` variable of the `glean` Nimbus
/// feature, or the `browser.newtabpage.ping.enabled` pref.
pub static newtab: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "newtab",
        true,
        false,
        vec!["component_init".into(), "newtab_session_end".into()],
    )
});

#[allow(non_upper_case_globals)]
/// A ping representing a single event happening with or to a TopSite.
/// Distinguishable by its `ping_type`.
/// Does not contain a `client_id`, preferring a `context_id` instead.
pub static top_sites: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "top-sites",
        false,
        false,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// Reinstrumentation of the Activity Stream "pocket-button" ping.
/// Submitted when actions are taken around the pocket button.
/// Does not contain any `client_id`.
/// Instead uses an `impression_id`.
pub static pocket_button: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "pocket-button",
        false,
        false,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// A ping representing a single event happening with or to a QuickSuggest.
/// Distinguishable by its `ping_type`.
/// Does not contain a `client_id`, preferring a `context_id` instead.
pub static quick_suggest: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "quick-suggest",
        false,
        false,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// Instrumentation collected during a page load.
pub static pageload: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "pageload",
        false,
        false,
        vec!["startup".into(), "threshold".into()],
    )
});

#[allow(non_upper_case_globals)]
/// This ping is generic for background tasks. Each background task can
/// gather its metrics under this ping and submit it when the task finishes.
/// Note that the ping submission must be done manually.
pub static background_tasks: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "background-tasks",
        false,
        false,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// A ping to report crash information. This information is sent as soon as
/// possible after a crash occurs (whether the crash is a background/content
/// process or the main process). It is expected to be used for crash report
/// analysis and to reduce blind spots in crash reporting.
pub static crash: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "crash",
        true,
        false,
        vec!["crash".into(), "event_found".into()],
    )
});

#[allow(non_upper_case_globals)]
/// This ping is for tests only.
pub static one_ping_only: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "one-ping-only",
        false,
        true,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// This ping is for tests only.
pub static test_ping: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "test-ping",
        false,
        true,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// Sent during startup if Firefox was launched by the installer.
pub static first_startup: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "first-startup",
        true,
        false,
        vec![],
    )
});

#[allow(non_upper_case_globals)]
/// This opt-out ping is sent from the Default Agent, which is a Windows-only
/// Firefox Background Task that is registered during Firefox installation with the
/// Windows scheduled tasks system so that it runs automatically every 24 hours,
/// whether Firefox is running or not.
/// Opting out of telemetry is handled via the pref value being copied to the
/// registry so that the Default Agent can read it without needing to work with
/// profiles. Relevant policies are consulted as well. The agent also has its own
/// pref, `default-agent.enabled`, which if set to false disables all agent
/// functionality, including generating this ping.
/// Each installation of Firefox has its own copy of the agent and its own
/// scheduled task which shares a common `LastPingSentAt` user registry key with
/// other installations. Installations race to send a single ping per 24 hour
/// window per installing user. If multiple operating system-level users are all
/// using one copy of Firefox, only one scheduled task will have been created and
/// only one ping will be sent, even though the users might have different default
/// browser settings. If multiple users have installed Firefox then each installing
/// user will have a scheduled task and ping.
/// Additional information for the Default Agent can be found in the [Default
/// Browser Agent docs](https://firefox-source-
/// docs.mozilla.org/toolkit/mozapps/defaultagent/default-browser-
/// agent/index.html).
pub static default_agent: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "default-agent",
        false,
        false,
        vec!["daily_ping".into()],
    )
});

#[allow(non_upper_case_globals)]
/// This ping measures the technical health of the background update system.
/// Said system downloads and processes updates when Firefox is not running.  It
/// is expected that this ping will be analyzed by humans to gain confidence in
/// the implementation as the staged rollout of the system proceeds to the
/// release channel, before settling into an automated analysis to detect spikes
/// in background update failure rates.  This ping will also help to
/// characterize the update-related settings of our user population.
/// 
/// Right now the background update system, and therefore this ping, is
/// restricted to Windows.
/// 
/// This ping is submitted only by the background update task.  It should be
/// submitted once per background update task invocation.  The expected schedule
/// is every 7 hours, controlled by the pref `app.update.background.interval`,
/// and subject to scheduling decisions made by the OS.
pub static background_update: Lazy<Ping> = Lazy::new(|| {
    Ping::new(
        "background-update",
        true,
        false,
        vec!["backgroundupdate_task".into()],
    )
});


/// Instantiate custom pings once to trigger registration.
///
/// # Arguments
///
/// application_id: If present, limit to only registering custom pings
///                 assigned to the identified application.
#[doc(hidden)]
pub fn register_pings(application_id: Option<&str>) {
    match application_id {
        Some("firefox.desktop") => {
            log::info!("Registering pings pageload, messaging-system, newtab, top-sites, pocket-button, quick-suggest, crash, first-startup, one-ping-only, test-ping for firefox.desktop");
            let _ = &*pageload;
            let _ = &*messaging_system;
            let _ = &*newtab;
            let _ = &*top_sites;
            let _ = &*pocket_button;
            let _ = &*quick_suggest;
            let _ = &*crash;
            let _ = &*first_startup;
            let _ = &*one_ping_only;
            let _ = &*test_ping;
        },
        Some("firefox.desktop.background.update") => {
            log::info!("Registering pings pageload, crash, background-update, one-ping-only, test-ping for firefox.desktop.background.update");
            let _ = &*pageload;
            let _ = &*crash;
            let _ = &*background_update;
            let _ = &*one_ping_only;
            let _ = &*test_ping;
        },
        _ => {
            let _ = &*messaging_system;
            let _ = &*newtab;
            let _ = &*top_sites;
            let _ = &*pocket_button;
            let _ = &*quick_suggest;
            let _ = &*pageload;
            let _ = &*background_tasks;
            let _ = &*crash;
            let _ = &*one_ping_only;
            let _ = &*test_ping;
            let _ = &*first_startup;
            let _ = &*default_agent;
            let _ = &*background_update;
        }
    }
}

#[cfg(feature = "with_gecko")]
pub(crate) fn submit_ping_by_id(id: u32, reason: Option<&str>) {
    if id & (1 << crate::factory::DYNAMIC_PING_BIT) > 0 {
        let map = crate::factory::__jog_metric_maps::PING_MAP
            .read()
            .expect("Read lock for dynamic ping map was poisoned!");
        if let Some(ping) = map.get(&id) {
            ping.submit(reason);
        } else {
            // TODO: instrument this error.
            log::error!("Cannot submit unknown dynamic ping {} by id.", id);
        }
        return;
    }
    match id {
        1 => messaging_system.submit(reason),
        2 => newtab.submit(reason),
        3 => top_sites.submit(reason),
        4 => pocket_button.submit(reason),
        5 => quick_suggest.submit(reason),
        6 => pageload.submit(reason),
        7 => background_tasks.submit(reason),
        8 => crash.submit(reason),
        9 => one_ping_only.submit(reason),
        10 => test_ping.submit(reason),
        11 => first_startup.submit(reason),
        12 => default_agent.submit(reason),
        13 => background_update.submit(reason),
        _ => {
            // TODO: instrument this error.
            log::error!("Cannot submit unknown ping {} by id.", id);
        }
    }
}
